@@ -73,14 +73,14 @@ def __virtual__():
     Only load if boto libraries exist.
     '''
     if not HAS_BOTO:
-        return False
+        return (False, 'The boto_kms module could not be loaded: boto libraries not found')
     return True
 
 
 def __init__(opts):
     salt.utils.compat.pack_dunder(__name__)
     if HAS_BOTO:
-        __utils__['boto.assign_funcs'](__name__, 'kms')
+        __utils__['boto.assign_funcs'](__name__, 'kms', pack=__salt__)
 
 
 def create_alias(alias_name, target_key_id, region=None, key=None, keyid=None,
