@@ -24,6 +24,7 @@ import datetime
 # Import 3rd-party libs
 # pylint: disable=import-error,redefined-builtin,no-name-in-module
 import salt.ext.six as six
+from salt.exceptions import SaltInvocationError
 import salt.utils.event
 from salt.ext.six.moves import configparser
 from salt.ext.six.moves.urllib.parse import urlparse as _urlparse
@@ -54,10 +55,10 @@ def __virtual__():
     Set the virtual pkg module if the os is openSUSE
     '''
     if __grains__.get('os_family', '') != 'Suse':
-        return False
+        return (False, "Module zypper: non SUSE OS not suppored by zypper package manager")
     # Not all versions of Suse use zypper, check that it is available
     if not salt.utils.which('zypper'):
-        return False
+        return (False, "Module zypper: zypper package manager not found")
     return __virtualname__
 
 
@@ -404,10 +405,10 @@ def info_available(*names, **kwargs):
 
     for pkg_data in pkg_info:
         nfo = {}
-        for line in [data for data in pkg_data.split("\n") if ":" in data]:
-            if line.startswith("-----"):
+        for line in [data for data in pkg_data.split('\n') if ':' in data]:
+            if line.startswith('-----'):
                 continue
-            kw = [data.strip() for data in line.split(":", 1)]
+            kw = [data.strip() for data in line.split(':', 1)]
             if len(kw) == 2 and kw[1]:
                 nfo[kw[0].lower()] = kw[1]
         if nfo.get('name'):
@@ -749,8 +750,10 @@ def mod_repo(repo, **kwargs):
 
             if new_url == base_url:
                 raise CommandExecutionError(
-                    'Repository \'{0}\' already exists as \'{1}\''
-                    .format(repo, alias)
+                    'Repository \'{0}\' already exists as \'{1}\'.'.format(
+                        repo,
+                        alias
+                    )
                 )
 
         # Add new repo
@@ -774,7 +777,9 @@ def mod_repo(repo, **kwargs):
         cmd_opt.append(kwargs['refresh'] and '--refresh' or '--no-refresh')
 
     if 'cache' in kwargs:
-        cmd_opt.append(kwargs['cache'] and '--keep-packages' or '--no-keep-packages')
+        cmd_opt.append(
+            kwargs['cache'] and '--keep-packages' or '--no-keep-packages'
+        )
 
     if 'gpgcheck' in kwargs:
         cmd_opt.append(kwargs['gpgcheck'] and '--gpgcheck' or '--no-gpgcheck')
@@ -836,6 +841,7 @@ def install(name=None,
             pkgs=None,
             sources=None,
             downloadonly=None,
+            skip_verify=False,
             version=None,
             **kwargs):
     '''
@@ -844,10 +850,10 @@ def install(name=None,
 
     name
         The name of the package to be installed. Note that this parameter is
-        ignored if either "pkgs" or "sources" is passed. Additionally, please
-        note that this option can only be used to install packages from a
-        software repository. To install a package file manually, use the
-        "sources" option.
+        ignored if either ``pkgs`` or ``sources`` is passed. Additionally,
+        please note that this option can only be used to install packages from
+        a software repository. To install a package file manually, use the
+        ``sources`` option.
 
         CLI Example:
 
@@ -866,10 +872,13 @@ def install(name=None,
     downloadonly
         Only download the packages, do not install.
 
+    skip_verify
+        Skip the GPG verification check (e.g., ``--no-gpg-checks``)
+
     version
         Can be either a version number, or the combination of a comparison
         operator (<, >, <=, >=, =) and a version number (ex. '>1.2.3-4').
-        This parameter is ignored if "pkgs" or "sources" is passed.
+        This parameter is ignored if ``pkgs`` or ``sources`` is passed.
 
 
     Multiple Package Installation Options:
@@ -951,14 +960,21 @@ def install(name=None,
     downgrades = []
     if fromrepo:
         fromrepoopt = ['--force', '--force-resolution', '--from', fromrepo]
-        log.info('Targeting repo {0!r}'.format(fromrepo))
+        log.info('Targeting repo \'{0}\''.format(fromrepo))
     else:
         fromrepoopt = ''
     cmd_install = ['install', '--name', '--auto-agree-with-licenses']
+    if not refresh:
+        cmd_install.append('--no-refresh')
+    if skip_verify:
+        cmd_install.append('--no-gpg-checks')
     if downloadonly:
         cmd_install.append('--download-only')
     if fromrepo:
         cmd_install.extend(fromrepoopt)
+
+    errors = []
+
     # Split the targets into batches of 500 packages each, so that
     # the maximal length of the command line is not broken
     while targets:
@@ -976,11 +992,18 @@ def install(name=None,
 
     __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
+    ret = salt.utils.compare_dicts(old, new)
 
-    return salt.utils.compare_dicts(old, new)
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered installing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 
-def upgrade(refresh=True):
+def upgrade(refresh=True, skip_verify=False):
     '''
     Run a full system upgrade, a zypper upgrade
 
@@ -999,6 +1022,13 @@ def upgrade(refresh=True):
     .. code-block:: bash
 
         salt '*' pkg.upgrade
+
+
+    Options:
+
+    skip_verify
+        Skip the GPG verification check (e.g., ``--no-gpg-checks``)
+
     '''
     ret = {'changes': {},
            'result': True,
@@ -1008,7 +1038,11 @@ def upgrade(refresh=True):
     if refresh:
         refresh_db()
     old = list_pkgs()
-    __zypper__.noraise.call('update', '--auto-agree-with-licenses')
+
+    to_append = ''
+    if skip_verify:
+        to_append = '--no-gpg-checks'
+    __zypper__.noraise.call('update', '--auto-agree-with-licenses', to_append)
     if __zypper__.exit_code not in __zypper__.SUCCESS_EXIT_CODES:
         ret['result'] = False
         ret['comment'] = (__zypper__.stdout() + os.linesep + __zypper__.stderr()).strip()
@@ -1035,12 +1069,21 @@ def _uninstall(name=None, pkgs=None):
     if not targets:
         return {}
 
+    errors = []
     while targets:
         __zypper__.call('remove', *targets[:500])
         targets = targets[500:]
-    __context__.pop('pkg.list_pkgs', None)
 
-    return salt.utils.compare_dicts(old, list_pkgs())
+    __context__.pop('pkg.list_pkgs', None)
+    ret = salt.utils.compare_dicts(old, list_pkgs())
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered removing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 
 def remove(name=None, pkgs=None, **kwargs):  # pylint: disable=unused-argument
@@ -1136,7 +1179,8 @@ def list_locks():
 
 def clean_locks():
     '''
-    Remove unused locks that do not currently (with regard to repositories used) lock any package.
+    Remove unused locks that do not currently (with regard to repositories
+    used) lock any package.
 
     CLI Example:
 
@@ -1347,7 +1391,6 @@ def _get_patterns(installed_only=None):
     List all known patterns in repos.
     '''
     patterns = {}
-
     for element in __zypper__.nolock.xml.call('se', '-t', 'pattern').getElementsByTagName('solvable'):
         installed = element.getAttribute('status') == 'installed'
         if (installed_only and installed) or not installed_only:
@@ -1413,7 +1456,9 @@ def search(criteria, refresh=False):
 
     solvables = __zypper__.nolock.xml.call('se', criteria).getElementsByTagName('solvable')
     if not solvables:
-        raise CommandExecutionError('No packages found by criteria "{0}".'.format(criteria))
+        raise CommandExecutionError(
+            'No packages found matching \'{0}\''.format(criteria)
+        )
 
     out = {}
     for solvable in [slv for slv in solvables
@@ -1503,7 +1548,7 @@ def list_products(all=False, refresh=False):
 
 
 def download(*packages, **kwargs):
-    """
+    '''
     Download packages to the local disk.
 
     refresh
@@ -1517,11 +1562,11 @@ def download(*packages, **kwargs):
 
         salt '*' pkg.download httpd
         salt '*' pkg.download httpd postfix
-    """
-    refresh = kwargs.get('refresh', False)
+    '''
     if not packages:
-        raise CommandExecutionError("No packages has been specified.")
+        raise SaltInvocationError('No packages specified')
 
+    refresh = kwargs.get('refresh', False)
     if refresh:
         refresh_db()
 
@@ -1529,16 +1574,20 @@ def download(*packages, **kwargs):
     for dld_result in __zypper__.xml.call('download', *packages).getElementsByTagName("download-result"):
         repo = dld_result.getElementsByTagName("repository")[0]
         pkg_info = {
-            'repository-name': repo.getAttribute("name"),
-            'repository-alias': repo.getAttribute("alias"),
-            'path': dld_result.getElementsByTagName("localfile")[0].getAttribute("path"),
+            'repository-name': repo.getAttribute('name'),
+            'repository-alias': repo.getAttribute('alias'),
         }
-        pkg_ret[_get_first_aggregate_text(dld_result.getElementsByTagName("name"))] = pkg_info
+        key = _get_first_aggregate_text(
+            dld_result.getElementsByTagName('name')
+        )
+        pkg_ret[key] = pkg_info
 
     if pkg_ret:
         return pkg_ret
 
-    raise CommandExecutionError("Unable to download packages: {0}.".format(', '.join(packages)))
+    raise CommandExecutionError(
+        'Unable to download packages: {0}'.format(', '.join(packages))
+    )
 
 
 def diff(*paths):
@@ -1572,6 +1621,9 @@ def diff(*paths):
         local_pkgs = __salt__['pkg.download'](*pkg_to_paths.keys())
         for pkg, files in six.iteritems(pkg_to_paths):
             for path in files:
-                ret[path] = __salt__['lowpkg.diff'](local_pkgs[pkg]['path'], path) or 'Unchanged'
+                ret[path] = __salt__['lowpkg.diff'](
+                    local_pkgs[pkg]['path'],
+                    path
+                ) or 'Unchanged'
 
     return ret
