@@ -454,6 +454,19 @@ def _compare(actual, create_kwargs, defaults_from_image):
     return ret
 
 
+def _find_volume(name):
+    '''
+    Find volume by name on minion
+    '''
+    docker_volumes = __salt__['dockerng.volumes']()['Volumes']
+    if docker_volumes:
+        volumes = [v for v in docker_volumes if v['Name'] == name]
+        if volumes:
+            return volumes[0]
+
+    return None
+
+
 def _get_defaults_from_image(image_id):
     return __salt__['dockerng.inspect_image'](image_id)
 
@@ -2241,10 +2254,8 @@ def volume_present(name, driver=None, driver_opts=None, force=False):
            'comment': ''}
     if salt.utils.is_dictlist(driver_opts):
         driver_opts = salt.utils.repack_dictlist(driver_opts)
-    volumes = __salt__['dockerng.volumes']()['Volumes']
-    if volumes is not None:
-        volumes = [v for v in volumes if v['Name'] == name]
-    if not volumes:
+    volume = _find_volume(name)
+    if not volume:
         try:
             ret['changes']['created'] = __salt__['dockerng.create_volume'](
                 name, driver=driver, driver_opts=driver_opts)
@@ -2257,7 +2268,6 @@ def volume_present(name, driver=None, driver_opts=None, force=False):
             ret['result'] = result
             return ret
     # volume exits, check if driver is the same.
-    volume = volumes[0]
     if driver is not None and volume['Driver'] != driver:
         if not force:
             ret['comment'] = "Driver for existing volume '{0}' ('{1}')" \
@@ -2311,8 +2321,8 @@ def volume_absent(name, driver=None):
            'result': False,
            'comment': ''}
 
-    volumes = [v for v in __salt__['dockerng.volumes']()['Volumes'] if v['Name'] == name]
-    if not volumes:
+    volume = _find_volume(name)
+    if not volume:
         ret['result'] = True
         ret['comment'] = 'Volume \'{0}\' already absent'.format(name)
         return ret
